@@ -3,7 +3,7 @@
 
 static gboolean run_func(ObActionsData *data, gpointer options);
 
-void action_close_startup()
+void action_close_startup(void)
 {
     actions_register("Close",
                      NULL, NULL,
