@@ -1,6 +1,7 @@
 #include "openbox/actions.h"
 #include "openbox/event.h"
 #include "openbox/startupnotify.h"
+#include "openbox/prompt.h"
 #include "openbox/screen.h"
 #include "gettext.h"
 
@@ -14,6 +15,7 @@ typedef struct {
     gchar   *sn_name;
     gchar   *sn_icon;
     gchar   *sn_wmclass;
+    gchar   *prompt;
 } Options;
 
 static gpointer setup_func(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node);
@@ -51,6 +53,9 @@ static gpointer setup_func(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node)
         g_free(s);
     }
 
+    if ((n = parse_find_node("prompt", node)))
+        o->prompt = parse_string(doc, n);
+
     if ((n = parse_find_node("startupnotify", node))) {
         xmlNodePtr m;
         if ((m = parse_find_node("enabled", n->xmlChildrenNode)))
@@ -74,10 +79,38 @@ static void free_func(gpointer options)
         g_free(o->sn_name);
         g_free(o->sn_icon);
         g_free(o->sn_wmclass);
+        g_free(o->prompt);
         g_free(o);
     }
 }
 
+static Options* dup_options(Options *in)
+{
+    Options *o = g_new(Options, 1);
+    o->cmd = g_strdup(in->cmd);
+    o->sn = in->sn;
+    o->sn_name = g_strdup(in->sn_name);
+    o->sn_icon = g_strdup(in->sn_icon);
+    o->sn_wmclass = g_strdup(in->sn_wmclass);
+    o->prompt = NULL;
+    return o;
+}
+
+static gboolean run_func(ObActionsData *data, gpointer options);
+
+static gboolean prompt_cb(ObPrompt *p, gint result, gpointer options)
+{
+    if (result)
+        run_func(NULL, options);
+    return TRUE; /* call the cleanup func */
+}
+
+static void prompt_cleanup(ObPrompt *p, gpointer options)
+{
+    prompt_unref(p);
+    free_func(options);
+}
+
 /* Always return FALSE because its not interactive */
 static gboolean run_func(ObActionsData *data, gpointer options)
 {
@@ -87,9 +120,26 @@ static gboolean run_func(ObActionsData *data, gpointer options)
     Options *o = options;
 
     if (!o->cmd) return FALSE;
+
+    if (o->prompt) {
+        ObPrompt *p;
+        Options *ocp;
+        ObPromptAnswer answers[] = {
+            { _("No"), 0 },
+            { _("Yes"), 1 }
+        };
+
+        ocp = dup_options(options);
+        p = prompt_new(o->prompt, _("Execute"), answers, 2, 0, 0,
+                       prompt_cb, prompt_cleanup, ocp);
+        prompt_show(p, NULL, FALSE);
+
+        return FALSE;
+    }
+
     cmd = g_filename_from_utf8(o->cmd, -1, NULL, NULL, NULL);
     if (!cmd) {
-        g_message(_("Failed to convert the path '%s' from utf8"), o->cmd);
+        g_message(_("Failed to convert the path \"%s\" from utf8"), o->cmd);
         return FALSE;
     }
 
@@ -98,7 +148,7 @@ static gboolean run_func(ObActionsData *data, gpointer options)
     event_cancel_all_key_grabs();
 
     if (!g_shell_parse_argv(cmd, NULL, &argv, &e)) {
-        g_message(_("Failed to execute '%s': %s"), o->cmd, e->message);
+        g_message(e->message, o->cmd);
         g_error_free(e);
     }
     else {
@@ -117,7 +167,7 @@ static gboolean run_func(ObActionsData *data, gpointer options)
                            G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
                            NULL, NULL, NULL, &e))
         {
-            g_message(_("Failed to execute '%s': %s"), o->cmd, e->message);
+            g_message(e->message, o->cmd);
             g_error_free(e);
 
             if (o->sn)
