@@ -3,7 +3,7 @@
 
 static gboolean run_func(ObActionsData *data, gpointer options);
 
-void action_unfocus_startup()
+void action_unfocus_startup(void)
 {
     actions_register("Unfocus", NULL, NULL, run_func, NULL, NULL);
 }
