@@ -73,7 +73,6 @@ ObClient* focus_cycle(gboolean forward, gboolean all_desktops,
                       gboolean showbar, gboolean dialog,
                       gboolean done, gboolean cancel)
 {
-    static ObClient *t = NULL;
     static GList *order = NULL;
     GList *it, *start, *list;
     ObClient *ft = NULL;
@@ -97,7 +96,6 @@ ObClient* focus_cycle(gboolean forward, gboolean all_desktops,
         list = client_list;
     }
 
-
     if (focus_cycle_target == NULL) {
         focus_cycle_iconic_windows = TRUE;
         focus_cycle_all_desktops = all_desktops;
@@ -150,7 +148,6 @@ ObClient* focus_cycle(gboolean forward, gboolean all_desktops,
 done_cycle:
     if (done && !cancel) ret = focus_cycle_target;
 
-    t = NULL;
     focus_cycle_target = NULL;
     g_list_free(order);
     order = NULL;
