@@ -62,7 +62,6 @@ static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
                                gchar **strippedlabel, guint *position,
                                gboolean *always_show);
 
-
 static void client_dest(ObClient *client, gpointer data)
 {
     /* menus can be associated with a client, so close any that are since
@@ -100,7 +99,7 @@ void menu_startup(gboolean reconfig)
             parse_tree(menu_parse_inst, doc, node->children);
             xmlFreeDoc(doc);
         } else
-            g_message(_("Unable to find a valid menu file '%s'"),
+            g_message(_("Unable to find a valid menu file \"%s\""),
                       (const gchar*)it->data);
     }
     if (!loaded) {
@@ -108,7 +107,7 @@ void menu_startup(gboolean reconfig)
             parse_tree(menu_parse_inst, doc, node->children);
             xmlFreeDoc(doc);
         } else
-            g_message(_("Unable to find a valid menu file '%s'"),
+            g_message(_("Unable to find a valid menu file \"%s\""),
                       "menu.xml");
     }
 
@@ -168,7 +167,7 @@ void menu_pipe_execute(ObMenu *self)
         return;
 
     if (!g_spawn_command_line_sync(self->execute, &output, NULL, NULL, &err)) {
-        g_message(_("Failed to execute command for pipe-menu '%s': %s"),
+        g_message(_("Failed to execute command for pipe-menu \"%s\": %s"),
                   self->execute, err->message);
         g_error_free(err);
         return;
@@ -182,7 +181,7 @@ void menu_pipe_execute(ObMenu *self)
         parse_tree(menu_parse_inst, doc, node->children);
         xmlFreeDoc(doc);
     } else {
-        g_message(_("Invalid output from pipe-menu '%s'"), self->execute);
+        g_message(_("Invalid output from pipe-menu \"%s\""), self->execute);
     }
 
     g_free(output);
@@ -195,7 +194,7 @@ static ObMenu* menu_from_name(gchar *name)
     g_assert(name != NULL);
 
     if (!(self = g_hash_table_lookup(menu_hash, name)))
-        g_message(_("Attempted to access menu '%s' but it does not exist"),
+        g_message(_("Attempted to access menu \"%s\" but it does not exist"),
                   name);
     return self;
 }
@@ -504,6 +503,7 @@ void menu_entry_unref(ObMenuEntry *self)
     if (self && --self->ref == 0) {
         switch (self->type) {
         case OB_MENU_ENTRY_TYPE_NORMAL:
+            RrImageUnref(self->data.normal.icon);
             g_free(self->data.normal.label);
             while (self->data.normal.actions) {
                 actions_act_unref(self->data.normal.actions->data);
