@@ -117,7 +117,7 @@ gboolean menu_frame_show_topmenu(ObMenuFrame *self, gint x, gint y,
 gboolean menu_frame_show_submenu(ObMenuFrame *self, ObMenuFrame *parent,
                                  ObMenuEntryFrame *parent_entry);
 
-void menu_frame_hide_all();
+void menu_frame_hide_all(void);
 void menu_frame_hide_all_client(struct _ObClient *client);
 
 void menu_frame_render(ObMenuFrame *self);
