@@ -51,6 +51,8 @@ typedef enum
     OB_KEY_RIGHT,
     OB_KEY_UP,
     OB_KEY_DOWN,
+    OB_KEY_TAB,
+    OB_KEY_SPACE,
     OB_NUM_KEYS
 } ObKey;
 
@@ -58,7 +60,8 @@ typedef enum
 {
     OB_STATE_STARTING,
     OB_STATE_RUNNING,
-    OB_STATE_EXITING
+    OB_STATE_EXITING,
+    OB_STATE_RECONFIGURING
 } ObState;
 
 typedef enum
