@@ -46,6 +46,7 @@
 #include "config.h"
 #include "ping.h"
 #include "mainloop.h"
+#include "prompt.h"
 #include "gettext.h"
 #include "parser/parse.h"
 #include "render/render.h"
@@ -84,18 +85,18 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
-
-RrInstance *ob_rr_inst;
-RrTheme    *ob_rr_theme;
-ObMainLoop *ob_main_loop;
-Display    *ob_display;
-gint        ob_screen;
-gboolean    ob_replace_wm = FALSE;
-gboolean    ob_sm_use = TRUE;
-gchar      *ob_sm_id = NULL;
-gchar      *ob_sm_save_file = NULL;
-gboolean    ob_sm_restore = TRUE;
-gboolean    ob_debug_xinerama = FALSE;
+RrInstance   *ob_rr_inst;
+RrImageCache *ob_rr_icons;
+RrTheme      *ob_rr_theme;
+ObMainLoop   *ob_main_loop;
+Display      *ob_display;
+gint          ob_screen;
+gboolean      ob_replace_wm = FALSE;
+gboolean      ob_sm_use = TRUE;
+gchar        *ob_sm_id = NULL;
+gchar        *ob_sm_save_file = NULL;
+gboolean      ob_sm_restore = TRUE;
+gboolean      ob_debug_xinerama = FALSE;
 
 static ObState   state;
 static gboolean  xsync = FALSE;
@@ -119,7 +120,7 @@ gint main(gint argc, gchar **argv)
 {
     gchar *program_name;
 
-    state = OB_STATE_STARTING;
+    ob_set_state(OB_STATE_STARTING);
 
     /* initialize the locale */
     if (!setlocale(LC_ALL, ""))
@@ -129,7 +130,7 @@ gint main(gint argc, gchar **argv)
     textdomain(PACKAGE_NAME);
 
     if (chdir(g_get_home_dir()) == -1)
-        g_message(_("Unable to change to home directory '%s': %s"),
+        g_message(_("Unable to change to home directory \"%s\": %s"),
                   g_get_home_dir(), g_strerror(errno));
 
     /* parse the command line args, which can change the argv[0] */
@@ -146,7 +147,6 @@ gint main(gint argc, gchar **argv)
         session_startup(argc, argv);
     }
 
-
     ob_display = XOpenDisplay(NULL);
     if (ob_display == NULL)
         ob_exit_with_error(_("Failed to open the display from the DISPLAY environment variable."));
@@ -181,6 +181,11 @@ gint main(gint argc, gchar **argv)
     ob_rr_inst = RrInstanceNew(ob_display, ob_screen);
     if (ob_rr_inst == NULL)
         ob_exit_with_error(_("Failed to initialize the obrender library."));
+    /* Saving 3 resizes of an RrImage makes a lot of sense for icons, as there
+       are generally 3 icon sizes needed: the titlebar icon, the menu icon,
+       and the alt-tab icon
+    */
+    ob_rr_icons = RrImageCacheNew(3);
 
     XSynchronize(ob_display, xsync);
 
@@ -216,12 +221,13 @@ gint main(gint argc, gchar **argv)
     cursors[OB_CURSOR_NORTHWEST] = load_cursor("top_left_corner",
                                                XC_top_left_corner);
 
-
     prop_startup(); /* get atoms values for the display */
     extensions_query_all(); /* find which extensions are present */
 
     if (screen_annex()) { /* it will be ours! */
         do {
+            ObPrompt *xmlprompt = NULL;
+
             modkeys_startup(reconfigure);
 
             /* get the keycodes for keys we use */
@@ -231,6 +237,8 @@ gint main(gint argc, gchar **argv)
             keys[OB_KEY_RIGHT] = modkeys_sym_to_code(XK_Right);
             keys[OB_KEY_UP] = modkeys_sym_to_code(XK_Up);
             keys[OB_KEY_DOWN] = modkeys_sym_to_code(XK_Down);
+            keys[OB_KEY_TAB] = modkeys_sym_to_code(XK_Tab);
+            keys[OB_KEY_SPACE] = modkeys_sym_to_code(XK_space);
 
             {
                 ObParseInst *i;
@@ -321,6 +329,7 @@ gint main(gint argc, gchar **argv)
             mouse_startup(reconfigure);
             menu_frame_startup(reconfigure);
             menu_startup(reconfigure);
+            prompt_startup(reconfigure);
 
             if (!reconfigure) {
                 guint32 xid;
@@ -357,15 +366,37 @@ gint main(gint argc, gchar **argv)
 
             reconfigure = FALSE;
 
-            state = OB_STATE_RUNNING;
+            ob_set_state(OB_STATE_RUNNING);
+
+            /* look for parsing errors */
+            {
+                xmlErrorPtr e = xmlGetLastError();
+                if (e) {
+                    gchar *m;
+
+                    m = g_strdup_printf(_("One or more XML syntax errors were found while parsing the Openbox configuration files.  See stdout for more information.  The last error seen was in file \"%s\" line %d, with message: %s"), e->file, e->line, e->message);
+                    xmlprompt =
+                        prompt_show_message(m, _("Openbox Syntax Error"), _("Close"));
+                    g_free(m);
+                    xmlResetError(e);
+                }
+            }
+
             ob_main_loop_run(ob_main_loop);
-            state = OB_STATE_EXITING;
+            ob_set_state(reconfigure ?
+                         OB_STATE_RECONFIGURING : OB_STATE_EXITING);
+
+            if (xmlprompt) {
+                prompt_unref(xmlprompt);
+                xmlprompt = NULL;
+            }
 
             if (!reconfigure) {
                 dock_remove_all();
                 client_unmanage_all();
             }
 
+            prompt_shutdown(reconfigure);
             menu_shutdown(reconfigure);
             menu_frame_shutdown(reconfigure);
             mouse_shutdown(reconfigure);
@@ -393,6 +424,7 @@ gint main(gint argc, gchar **argv)
     XSync(ob_display, FALSE);
 
     RrThemeFree(ob_rr_theme);
+    RrImageCacheUnref(ob_rr_icons);
     RrInstanceFree(ob_rr_inst);
 
     session_shutdown(being_replaced);
@@ -413,7 +445,7 @@ gint main(gint argc, gchar **argv)
                 g_strfreev(argvp);
             } else {
                 g_message(
-                    _("Restart failed to execute new executable '%s': %s"),
+                    _("Restart failed to execute new executable \"%s\": %s"),
                     restart_path, err->message);
                 g_error_free(err);
             }
@@ -480,7 +512,7 @@ static void signal_handler(gint signal, gpointer data)
     }
 }
 
-static void print_version()
+static void print_version(void)
 {
     g_print("Openbox %s\n", PACKAGE_VERSION);
     g_print(_("Copyright (c)"));
@@ -492,13 +524,16 @@ static void print_version()
     g_print("under certain conditions. See the file COPYING for details.\n\n");
 }
 
-static void print_help()
+static void print_help(void)
 {
     g_print(_("Syntax: openbox [options]\n"));
     g_print(_("\nOptions:\n"));
     g_print(_("  --help              Display this help and exit\n"));
     g_print(_("  --version           Display the version and exit\n"));
     g_print(_("  --replace           Replace the currently running window manager\n"));
+    /* TRANSLATORS: if you translate "FILE" here, make sure to keep the "Specify..."
+       aligned still, if you have to, make a new line with \n and 22 spaces. It's
+       fine to leave it as FILE though. */
     g_print(_("  --config-file FILE  Specify the path to the config file to use\n"));
     g_print(_("  --sm-disable        Disable connection to the session manager\n"));
     g_print(_("\nPassing messages to a running Openbox instance:\n"));
@@ -524,7 +559,7 @@ static void remove_args(gint *argc, gchar **argv, gint index, gint num)
     *argc -= num;
 }
 
-static void parse_env()
+static void parse_env(void)
 {
     /* unset this so we don't pass it on unknowingly */
     unsetenv("DESKTOP_STARTUP_ID");
@@ -622,7 +657,7 @@ static void parse_args(gint *argc, gchar **argv)
         else {
             /* this is a memleak.. oh well.. heh */
             gchar *err = g_strdup_printf
-                (_("Invalid command line argument '%s'\n"), argv[i]);
+                (_("Invalid command line argument \"%s\"\n"), argv[i]);
             ob_exit_with_error(err);
         }
     }
@@ -653,13 +688,13 @@ void ob_restart_other(const gchar *path)
     ob_restart();
 }
 
-void ob_restart()
+void ob_restart(void)
 {
     restart = TRUE;
     ob_exit(0);
 }
 
-void ob_reconfigure()
+void ob_reconfigure(void)
 {
     reconfigure = TRUE;
     ob_exit(0);
@@ -671,7 +706,7 @@ void ob_exit(gint code)
     ob_main_loop_exit(ob_main_loop);
 }
 
-void ob_exit_replace()
+void ob_exit_replace(void)
 {
     exitcode = 0;
     being_replaced = TRUE;
@@ -690,7 +725,12 @@ KeyCode ob_keycode(ObKey key)
     return keys[key];
 }
 
-ObState ob_state()
+ObState ob_state(void)
 {
     return state;
 }
+
+void ob_set_state(ObState s)
+{
+    state = s;
+}
