@@ -51,6 +51,7 @@ void prop_startup(void)
     CREATE(wm_client_machine, "WM_CLIENT_MACHINE");
     CREATE(wm_command, "WM_COMMAND");
     CREATE(wm_client_leader, "WM_CLIENT_LEADER");
+    CREATE(wm_transient_for, "WM_TRANSIENT_FOR");
     CREATE(motif_wm_hints, "_MOTIF_WM_HINTS");
 
     CREATE(sm_client_id, "SM_CLIENT_ID");
@@ -112,6 +113,7 @@ void prop_startup(void)
     CREATE(net_wm_window_type_splash, "_NET_WM_WINDOW_TYPE_SPLASH");
     CREATE(net_wm_window_type_dialog, "_NET_WM_WINDOW_TYPE_DIALOG");
     CREATE(net_wm_window_type_normal, "_NET_WM_WINDOW_TYPE_NORMAL");
+    CREATE(net_wm_window_type_popup_menu, "_NET_WM_WINDOW_TYPE_POPUP_MENU");
 
     prop_atoms.net_wm_moveresize_size_topleft = 0;
     prop_atoms.net_wm_moveresize_size_top = 1;
