@@ -33,6 +33,7 @@ GSList *session_desktop_names = NULL;
 void session_startup(gint argc, gchar **argv) {}
 void session_shutdown(gboolean permanent) {}
 GList* session_state_find(struct _ObClient *c) { return NULL; }
+void session_request_logout(gboolean silent) {}
 #else
 
 #include "debug.h"
@@ -100,7 +101,7 @@ void session_startup(gint argc, gchar **argv)
     dir = g_build_filename(parse_xdg_data_home_path(),
                            "openbox", "sessions", NULL);
     if (!parse_mkdir_path(dir, 0700)) {
-        g_message(_("Unable to make directory '%s': %s"),
+        g_message(_("Unable to make directory \"%s\": %s"),
                   dir, g_strerror(errno));
     }
 
@@ -155,7 +156,7 @@ void session_shutdown(gboolean permanent)
 }
 
 /*! Connect to the session manager and set up our callback functions */
-static gboolean session_connect()
+static gboolean session_connect(void)
 {
     SmcCallbacks cb;
     gchar *oldid;
@@ -189,7 +190,7 @@ static gboolean session_connect()
     return sm_conn != NULL;
 }
 
-static void session_setup_program()
+static void session_setup_program(void)
 {
     SmPropValue vals = {
         .value = sm_argv[0],
@@ -208,7 +209,7 @@ static void session_setup_program()
     g_free(prop.type);
 }
 
-static void session_setup_user()
+static void session_setup_user(void)
 {
     char *user = g_strdup(g_get_user_name());
 
@@ -251,7 +252,7 @@ static void session_setup_restart_style(gboolean restart)
     g_free(prop.type);
 }
 
-static void session_setup_pid()
+static void session_setup_pid(void)
 {
     gchar *pid = g_strdup_printf("%ld", (glong) getpid());
 
@@ -274,7 +275,7 @@ static void session_setup_pid()
 }
 
 /*! This is a gnome-session-manager extension */
-static void session_setup_priority()
+static void session_setup_priority(void)
 {
     gchar priority = 20; /* 20 is a lower prioity to run before other apps */
 
@@ -295,7 +296,7 @@ static void session_setup_priority()
     g_free(prop.type);
 }
 
-static void session_setup_clone_command()
+static void session_setup_clone_command(void)
 {
     gint i;
 
@@ -321,7 +322,7 @@ static void session_setup_clone_command()
     g_free(vals);
 }
 
-static void session_setup_restart_command()
+static void session_setup_restart_command(void)
 {
     gint i;
 
@@ -363,7 +364,7 @@ static void session_setup_restart_command()
     g_free(vals);
 }
 
-static ObSMSaveData *sm_save_get_data()
+static ObSMSaveData *sm_save_get_data(void)
 {
     ObSMSaveData *savedata = g_new0(ObSMSaveData, 1);
     /* save the active desktop and client.
@@ -396,14 +397,28 @@ static void sm_save_yourself_2(SmcConn conn, SmPointer data)
     SmcSaveYourselfDone(conn, success);
 }
 
-
 static void sm_save_yourself(SmcConn conn, SmPointer data, gint save_type,
                              Bool shutdown, gint interact_style, Bool fast)
 {
     ObSMSaveData *savedata = NULL;
     gchar *vendor;
 
-    ob_debug_type(OB_DEBUG_SM, "Session save requested\n");
+#ifdef DEBUG
+    {
+        const gchar *sname =
+            (save_type == SmSaveLocal ? "SmSaveLocal" :
+             (save_type == SmSaveGlobal ? "SmSaveGlobal" :
+              (save_type == SmSaveBoth ? "SmSaveBoth" : "INVALID!!")));
+        ob_debug_type(OB_DEBUG_SM, "Session save requested, type %s\n", sname);
+    }
+#endif
+
+    if (save_type == SmSaveGlobal) {
+        /* we have no data to save.  we only store state to get back to where
+           we were, we don't keep open writable files or anything */
+        SmcSaveYourselfDone(conn, TRUE);
+        return;
+    }
 
     vendor = SmcVendor(sm_conn);
     ob_debug_type(OB_DEBUG_SM, "Session manager's vendor: %s\n", vendor);
@@ -448,7 +463,7 @@ static gboolean session_save_to_file(const ObSMSaveData *savedata)
     f = fopen(ob_sm_save_file, "w");
     if (!f) {
         success = FALSE;
-        g_message(_("Unable to save the session to '%s': %s"),
+        g_message(_("Unable to save the session to \"%s\": %s"),
                   ob_sm_save_file, g_strerror(errno));
     } else {
         fprintf(f, "<?xml version=\"1.0\"?>\n\n");
@@ -580,7 +595,7 @@ static gboolean session_save_to_file(const ObSMSaveData *savedata)
 
         if (fflush(f)) {
             success = FALSE;
-            g_message(_("Error while saving the session to '%s': %s"),
+            g_message(_("Error while saving the session to \"%s\": %s"),
                       ob_sm_save_file, g_strerror(errno));
         }
         fclose(f);
@@ -804,4 +819,20 @@ static void session_load_file(const gchar *path)
     xmlFreeDoc(doc);
 }
 
+void session_request_logout(gboolean silent)
+{
+    if (sm_conn) {
+        SmcRequestSaveYourself(sm_conn,
+                               SmSaveGlobal,
+                               TRUE, /* logout */
+                               (silent ?
+                                SmInteractStyleNone : SmInteractStyleAny),
+                               TRUE,  /* if false, with GSM, it shows the old
+                                         logout prompt */
+                               TRUE); /* global */
+    }
+    else
+        g_message(_("Not connected to a session manager"));
+}
+
 #endif
