@@ -25,11 +25,16 @@
 #include "group.h"
 #include "frame.h"
 #include "window.h"
+#include "event.h"
 #include "debug.h"
 
 GList  *stacking_list = NULL;
+/*! When true, stacking changes will not be reflected on the screen.  This is
+  to freeze the on-screen stacking order while a window is being temporarily
+  raised during focus cycling */
+static gboolean pause_changes = FALSE;
 
-void stacking_set_list()
+void stacking_set_list(void)
 {
     Window *windows = NULL;
     GList *it;
@@ -99,12 +104,59 @@ static void do_restack(GList *wins, GList *before)
     }
 #endif
 
-    XRestackWindows(ob_display, win, i);
+    if (!pause_changes)
+        XRestackWindows(ob_display, win, i);
     g_free(win);
 
     stacking_set_list();
 }
 
+void stacking_temp_raise(ObWindow *window)
+{
+    Window win[2];
+    GList *it;
+    gulong start;
+
+    /* don't use this for internal windows..! it would lower them.. */
+    g_assert(window_layer(window) < OB_STACKING_LAYER_INTERNAL);
+
+    /* find the window to drop it underneath */
+    win[0] = screen_support_win;
+    for (it = stacking_list; it; it = g_list_next(it)) {
+        ObWindow *w = it->data;
+        if (window_layer(w) >= OB_STACKING_LAYER_INTERNAL)
+            win[0] = window_top(w);
+        else
+            break;
+    }
+
+    win[1] = window_top(window);
+    start = event_start_ignore_all_enters();
+    XRestackWindows(ob_display, win, 2);
+    event_end_ignore_all_enters(start);
+
+    pause_changes = TRUE;
+}
+
+void stacking_restore(void)
+{
+    Window *win;
+    GList *it;
+    gint i;
+    gulong start;
+
+    win = g_new(Window, g_list_length(stacking_list) + 1);
+    win[0] = screen_support_win;
+    for (i = 1, it = stacking_list; it; ++i, it = g_list_next(it))
+        win[i] = window_top(it->data);
+    start = event_start_ignore_all_enters();
+    XRestackWindows(ob_display, win, i);
+    event_end_ignore_all_enters(start);
+    g_free(win);
+
+    pause_changes = FALSE;
+}
+
 static void do_raise(GList *wins)
 {
     GList *it;
@@ -169,6 +221,15 @@ static void restack_windows(ObClient *selected, gboolean raise)
     GList *modals = NULL;
     GList *trans = NULL;
 
+    if (raise) {
+        ObClient *p;
+
+        /* if a window is modal for another single window, then raise it to the
+           top too, the same is done with the focus order */
+        while (selected->modal && (p = client_direct_parent(selected)))
+            selected = p;
+    }
+
     /* remove first so we can't run into ourself */
     it = g_list_find(stacking_list, selected);
     g_assert(it);
@@ -411,6 +472,7 @@ void stacking_add_nonintrusive(ObWindow *win)
     ObClient *client;
     GList *it_below = NULL; /* this client will be below us */
     GList *it_above;
+    GList *wins;
 
     if (!WINDOW_IS_CLIENT(win)) {
         stacking_add(win); /* no special rules for others */
@@ -468,7 +530,7 @@ void stacking_add_nonintrusive(ObWindow *win)
             break;
     }
 
-    GList *wins = g_list_append(NULL, win);
+    wins = g_list_append(NULL, win);
     do_restack(wins, it_below);
     g_list_free(wins);
 }
